

class FileLogger:

    def __init__(self, file_path):
        self.file_path = file_path
    
    def log(self, data):
        string = ' '.join(["%s=%s" % (key, value) for key, value in data.items()])
        string += '\r\n'
        self.write(string)

    # TODO: make write function async <<<
    def write(self, string):
        """ Not async :(( """
        with open(self.file_path, "a") as logfile:
            logfile.write(string)


import asyncio
import json


class AgentManager(asyncio.Protocol):
	'''
	ask for password and send password to trust each other
	'''

	state = 0

	def __init__(self, manager):
		self.manager = manager

	def connection_made(self, transport):
		self.transport = transport
		if not self.transport.get_extra_info('peername')[0] in self.manager.agents:
			# TODO: Log Stupid Connections
			self.transport.close()
		else:
			self.transport = transport
			data = b"Hello Agent!"
			if self.manager.agent_password:
				data +=  b' Give Me The Night Word'
			else:
				self.state = 1
			self.transport.write(data)

	def data_received(self, data):
		if not self.transport.get_extra_info('peername')[0] in self.manager.agents:
			return		
		if self.state == 0: # getting password
			if self.manager.agent_password == data:
				self.transport.write('Now Tell Me The Report')
				self.state = 1
		elif self.state == 1:
			data = json.loads(data.decode())
			self.manager.file_logger.log(data)
			self.transport.close()


from aiohttp import web

from demonhunter.nodes.manager.app import add_routes, run_app
from demonhunter.nodes.manager.agent.server import AgentManager
from demonhunter.core.loggers.logfile import FileLogger


class Manager:

    agents = list()

    def __init__(self, loop, agent_manager=True, web_app=None, agent_password=None, logfile=None):
        self.loop = loop

        if agent_manager:
            coro = self.loop.create_server(lambda: AgentManager(self), '127.0.0.1', 16742)
            server = self.loop.run_until_complete(coro)
            print('AgentManager Serving on {0}'.format(server.sockets[0].getsockname()))
            self.agent_password = agent_password

        if not web_app:
            app = web.Application(loop=self.loop)
            add_routes(app)
            run_app(app)

        if logfile:
            self.file_logger = FileLogger(logfile)


    def add_agent_address(self, address):
        self.agents.append(address)


import asyncio

from demonhunter.nodes.manager import Manager


class DemonHunter:

    honeypots = list()
    servers = list()
    agents = list()

    def __init__(self, loop):
        self.loop = loop

    def add_honeypot(self, honeypot):
        self.honeypots.append(honeypot)

    def add_agent(self, agent):
        self.agents.append(agent)

    def start(self):
        for honeypot in self.honeypots:
            for interface in honeypot.interfaces:
                coro = self.loop.create_server(lambda: honeypot.handler(honeypot), interface, honeypot.port)
                server = self.loop.run_until_complete(coro)
                self.servers.append(server)
                print('Serving on {0}'.format(server.sockets[0].getsockname()))

    def stop(self):
        for server in self.servers:
            server.close()


class Data:

    value = None

    def __init__(self, value, type, default=None):
        if not default == None:
            self.value = default
            self.type = 

    def set_value(self, value):
        self.value = value


class Model:

    def get_fields(self):
        for field in dir(self):
            if type(getattr(self, field)) == Data:
                print("Found one")


if __name__ == "__main__":

    class SSHAuth(Model):

        ip = Data()
        username = Data()
        password = Data()
        time = Data()

    test = SSHAuth()
    test.get_fields()


import asyncio
import json


class BaseHandler:
    # https://svn.nmap.org/nmap/nmap-service-probes

    def save_data(self, data):
        if self.honeypot.sqlite:
            self.save_in_sqlite(data)
        if self.honeypot.syslog:
            self.save_logfile(data)
        self.alter_agents(data)

    def alter_agents(self, data):
        for agent in self.honeypot.agents:
            agent.send_data(data)

    def save_in_sqlite(data):
        pass

    def save_logfile(data):
        pass


class Agent():

    port = 16742

    def __init__(self, target, honeypots, loop, agent_password=None):
        self.loop = loop
        self.targets = target
        for honeypot in honeypots:
            honeypot.agents.append(self)
        self.agent_password = None if not agent_password else agent_password.encode()

    # TODO: Secure the transport ssl/or something.
    def send_data(self, data):
        data = json.dumps(data)
        for target in self.targets:
            # i guess its not a good way :/
            coro = self.loop.create_connection(lambda: AgentProtocol(data, self.agent_password),
                                               target, 16742)
            self.loop.call_soon_threadsafe(asyncio.async, coro)


class AgentProtocol(asyncio.Protocol):

    state = 0

    def __init__(self, message, agent_password):
        self.message = message
        self.agent_password = agent_password

    def connection_made(self, transport):
        self.transport = transport

    def data_received(self, data):
        if data == b"Hello Agent!":
            self.send_data()
        elif data == b"Hello Agent! Give Me The Night Word":
            if self.agent_password:
                self.transport.write(self.agent_password)
            else:
                print("AgentManager Asks for password !!!??? did you forget to set a password ?")
                self.transport.close()

    def send_data(self):
        self.transport.write(str.encode(self.message))

    def connection_lost(self, exc):
        pass


from setuptools import setup, find_packages

long_description = """
DemonHunter is a framework to create a Honeypot network very simple and easy.
"""

setup(
    name='demonhunter',
    version='1.0.0',

    description='A Distributed Honeypot',
    long_description=long_description,

    url='https://github.com/RevengeComing/DemonHunter',
    author='Sepehr Hamzelooy',
    author_email='s.hamzelooy@gmail.com',
    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.5',
    ],
    install_requires=['aiohttp'],
    packages=find_packages(),

    keywords='honeypot honeynet agent',
)


from demonhunter.nodes.honeypots import BaseHandler

import asyncio
import time

from io import BytesIO

from telnetlib import (
    LINEMODE, NAWS, NEW_ENVIRON, BINARY, SGA, ECHO, STATUS,
    TTYPE, TSPEED, LFLOW, XDISPLOC, IAC, DONT, DO, WONT,
    WILL, SE, NOP, TM, DM, BRK, IP, AO, AYT, EC, EL, EOR,
    GA, SB, LOGOUT, CHARSET, SNDLOC, theNULL,
    # not supported or used,
    ENCRYPT, AUTHENTICATION, TN3270E, XAUTH, RSP,
    COM_PORT_OPTION, SUPPRESS_LOCAL_ECHO, TLS, KERMIT,
    SEND_URL, FORWARD_X, PRAGMA_LOGON, SSPI_LOGON,
    PRAGMA_HEARTBEAT, EXOPL, X3PAD, VT3270REGIME, TTYLOC,
    SUPDUPOUTPUT, SUPDUP, DET, BM, XASCII, RCP, NAMS,
    RCTE, NAOL, NAOP, NAOCRD, NAOHTS, NAOHTD, NAOFFD,
    NAOVTS, NAOVTD, NAOLFD)


iacBytes = {
    DO:   'DO',
    DONT: 'DONT',
    WILL: 'WILL',
    WONT: 'WONT',
    IP:   'IP'
}


class TelnetHandler(asyncio.Protocol, BaseHandler):
    """
    Copied twisted.protocols.telnet.Telnet mechanism.
    """

    state = 0
    next_state = None

    username = None
    login_tries = 0
    authenticated = False

    gotIAC = 0
    iacByte = None
    lastLine = None
    buffer = b''
    echo = 0
    delimiters = [b'\r\n', b'\r\000']
    mode = "User"

    def __init__(self, honeypot):
        self.honeypot = honeypot

    def connection_made(self, transport):
        self.honeypot.active_attacks += 1
        self.transport = transport
        print("Connection from {0}".format(self.transport.get_extra_info('peername')[0]))
        self.write(self.welcome_message() + self.login_prompt())

    def write(self, data):
        """Send the given data over my transport."""
        self.transport.write(data)

    def welcome_message(self):
        return b"Welcome To Telnet Server\r\n\n"

    def login_prompt(self):
        return b"username: "

    def iacSBchunk(self, chunk):
        pass

    def iac_DO(self, feature):
        pass

    def iac_DONT(self, feature):
        pass

    def iac_WILL(self, feature):
        pass

    def iac_WONT(self, feature):
        pass

    def iac_IP(self, feature):
        pass

    def processLine(self, line):
        mode = getattr(self, "telnet_" + self.mode)(line)
        if mode is not None:
            self.mode = mode

    def telnet_User(self, user):
        self.username = user
        self.write(IAC + WILL + ECHO + b"password: ")
        return "Password"

    def telnet_Password(self, paswd):
        self.write(IAC + WONT + ECHO + b"\r\n")
        self.write(b"\r\nFail login\r\n")

        self.save_authentication(paswd)

        self.login_tries += 1
        if self.login_tries == 3:
            return "Done"
        else:
            if self.login_tries != 1:
                self.write(b'\r\n')
            self.write(self.login_prompt())
            return "User"

    def process_chunk(self, chunk):
        self.buffer = self.buffer + chunk

        for delim in self.delimiters:
            idx = self.buffer.find(delim)
            if idx != -1:
                break

        while idx != -1:
            buf, self.buffer = self.buffer[:idx], self.buffer[idx+2:]
            self.processLine(buf)
            if self.mode == 'Done':
                self.transport.close()

            for delim in self.delimiters:
                idx = self.buffer.find(delim)
                if idx != -1:
                    break

    def data_received(self, data):
        chunk = BytesIO()

        def iterbytes(originalBytes):
            for i in range(len(originalBytes)):
                yield originalBytes[i:i+1]

        for char in iterbytes(data):
            if self.gotIAC:
                if self.iacByte:
                    if self.iacByte == SB:
                        if char == SE:
                            self.iacSBchunk(chunk.getvalue())
                            chunk = BytesIO()
                            del self.iacByte
                            del self.gotIAC
                        else:
                            chunk.write(char)
                    else:
                        try:
                            getattr(self, 'iac_%s' % iacBytes[self.iacByte])(char)
                        except KeyError:
                            pass
                        del self.iacByte
                        del self.gotIAC
                else:
                    self.iacByte = char
            elif char == IAC:

                c = chunk.getvalue()
                if c:
                    why = self.process_chunk(c)
                    if why:
                        return why
                    chunk = BytesIO()
                self.gotIAC = 1
            else:
                chunk.write(char)

        c = chunk.getvalue()
        if c:
            why = self.process_chunk(c)
            if why:
                return why

    def set_username(self, data):
        self.username = data
        self.transport.write(IAC + WILL + ECHO)
        self.transport.write(b"password: ")

    def save_authentication(self, password):
        attack_time = time.time()
        print('Attack from {0}, with username {1}, and password {2}.'.format(self.transport.get_extra_info('peername')[0],
                                                                             str(self.username),
                                                                             str(password)))
        data = {'protocol':'telnet', 'from':self.transport.get_extra_info('peername')[0],
                'username':self.username.decode('utf-8'), 'password':password.decode('utf-8'), 'time':attack_time}
        self.save_data(data)

    def connection_lost(self, exc):
        print("Connection lost from {0}".format(self.transport.get_extra_info('peername')[0]))
        self.honeypot.active_attacks -= 1


class DebianTelnet(TelnetHandler):
    def welcome_message(self):
        return b"Debian GNU/Linux 7\r\n\n"

    def login_prompt(self):
        return b"Login: "


class MicrosoftTelnet(TelnetHandler):

    def welcome_message(self):
        return b"Welcome to Microsoft Telnet Service\r\n\n"

    def login_prompt(self):
        return b"login: "


class TelnetHoneypot:

    syslog = False
    sqlite = False
    agents = []

    active_attacks = 0

    interfaces = ['0.0.0.0']
    port = 23

    handler = TelnetHandler


import asyncio

from aiohttp import web
from aiohttp.log import access_logger

@asyncio.coroutine
def index(request):
    text = "This is will be somewhere u can see DemonHunter's Reports."
    return web.Response(text=text)

def add_routes(app):
	app.router.add_get('/', index)

def run_app(app, *, host='0.0.0.0', port=None,
            shutdown_timeout=60.0, ssl_context=None,
            print=print, backlog=128, access_log_format=None,
            access_log=access_logger):
    """ Copied From aiohttp's main run_app """

    if port is None:
        if not ssl_context:
            port = 8080
        else:
            port = 8443

    loop = app.loop

    make_handler_kwargs = dict()
    if access_log_format is not None:
        make_handler_kwargs['access_log_format'] = access_log_format
    handler = app.make_handler(access_log=access_log,
                               **make_handler_kwargs)
    server = loop.create_server(handler, host, port, ssl=ssl_context,
                                backlog=backlog)
    srv, startup_res = loop.run_until_complete(asyncio.gather(server,
                                                              app.startup(),
                                                              loop=loop))

    scheme = 'https' if ssl_context else 'http'
    url = '{}://localhost:{}'.format(scheme, port)
    print("DemonHunter Manager Web App Running on {}".format(url))